
/*********************************************************************************************************
  The program reads the radio weather forecast from hamsql.com and iap-kühlungsborn and outputs it to a display
  the datas are from "hamsql.com" by OM Paul Herrman , N0NBH -> THANKS a lot for it !!
  Thanks also to "Institut für Ionophärenphysik Rostock e.V. Aussenstelle Kühlungsborn" , Mr.Jens Mielich
  Hardware: M5Stack core 2 - no more !
  program idea and realization by Dr. Peter Heblik , DD6USB , version 1.8 from 6/16/2022
  "OTA"  means: the D1 mini can be programmed via WLAN - it does not have to be connected to a computer.
  how to program: in Arduino-IDE you find -> "Tools" -> "Ports" the Adress from "M5Stack for Solardat at" and his local IP.
  Select it and download - thats all folks ! But attention! in OTA-mode the serial monitor does NOT work !!!*/
//=========================================================================================================
//     =====>>> put in lines 30/31 your WiFi credentials <<<=====
//=========================================================================================================
/**********************************************************************************************************/

//#define debug           //  scratch out if not used
//#define debug1          //  deep debug,scratch out if not used
//#define debug2          //  special debug, scratch out if not used
#define WiFiOn            // to switch WiFi & WWW on/off (for faster screen-tests)

#include <Arduino.h>
#include <M5Core2.h>
#include <WiFi.h>
#include <WiFiMulti.h>
#include <HTTPClient.h>
#include <ArduinoOTA.h>
#include <colors.h>

//include here your WLAN credentials (ssid,passw)  // WLAN-datas (individual)
 const char * ssid =             // for instance "AREA51"
 const char * passw =            // for instance "11223344"

WiFiMulti wifiMulti ;
TFT_eSprite img = TFT_eSprite(&M5.lcd) ;

#define HOST "https://www.hamqsl.com/solarxml.php" // source page in www
#define HOST1 "https://www.ionosonde.iap-kborn.de/latest_JR055_MUF.txt" // another source page in www
#define max_page 4                        // only 5 pages, no more (at this time)[p0->p4]

//============>>   here the global variables: <<=============

const char * V_String = "M5Stack_Solar v1.8, 6/16/2022 by DD6USB" ;
char atom ;                           // single char of content
int page_nr , old_page_nr ;           // which page I am ?
uint16_t color , x , k , pos , pos1 ; // for band-field-colors and string-positions
uint16_t client_bytes = 1700 ;        // max. bytes to read from content
uint64_t start_time ;                 // for time_loop
uint64_t loop_time = 900000 ;         // 900.000 milli-secs = 15 min
bool is_basic_pattern ;               // flag for page0-pattern
bool flag_color = false ;             // flag for color change or not
bool ext , old_ext = true ;           // flag for extern or LiPo powered
String pbuf, old_pbuf, t, t2 ;        // to hold the old and new compressed content
String old_iap , new_iap , iap_buf ;  // hold the iap-datas for comparison
String content_strings3[10] ;         // hold the iap-datas
float Percentage , old_Percentage ;   // LiPo loading status

//***********************************************************************
void setup(void)
{
#ifdef debug
#ifdef debug1
#ifdef debug2
  Serial.begin(115200) ;
  delay(500) ;
  Serial.println(V_String);
#endif
#endif
#endif
  M5.begin() ;                     //Init M5Core2
  delay(300) ;
  check_voltage() ;               // if LiPo discharged : switch device off !!!
  M5.Lcd.begin() ;
  M5.Lcd.setRotation(0) ;
#ifdef WiFiOn
  wifiMulti.addAP(ssid, passw) ;   //Connect wifi and return connection status
  M5.Lcd.fillScreen(BLUE) ;
  M5.Lcd.setTextColor(WHITE) ;
  M5.Lcd.setTextSize(2) ;
  M5.Lcd.println("\nWaiting Wifi Connect") ;
  while (wifiMulti.run() != WL_CONNECTED)
  {
    delay(100);
    M5.lcd.print(".") ;
  }
  M5.lcd.println("\n==>> WiFi Connected!") ;
  delay(500) ;
  M5.lcd.println("\n....initialize OTA") ;
  delay(500) ;
  handle_OTA() ;                 // initialize OTA
  M5.lcd.println("\n.... OTA o.k.") ;
  delay(500) ;
#endif
  delay(500) ;
  ext = M5.Axp.isACIN() ;        // extern powered ?
  old_ext = ext ;
  M5.lcd.setFont() ;
  page_nr = 0  ;                // start-page
  handle_pages() ;              // go a "first round"
  start_time = millis() ;       // start timeloop
}
//***************************************************************
void loop(void)
{
  delay(100) ;
  M5.update() ;
  check_voltage() ;               // if LiPo discharged : switch device off !!!
#ifdef WiFiOn
  ArduinoOTA.handle();            // occur new OTA programming ?
#endif
  if (M5.Touch.ispressed())
  {
    page_nr == max_page ? page_nr = 0 : page_nr++ ; // revolving pages
    handle_pages() ;
    delay(500) ;
  }
  if (time_loop_finished())
  {
    if ((new_iap != old_iap) || !check_N0NBH_datas())
    {
#ifdef debug
      Serial.println("old_iap = " + old_iap) ;
      Serial.println("new_iap = " + new_iap) ;
      Serial.println(check_N0NBH_datas()) ;
#endif
      old_iap = new_iap ;
      handle_pages() ;
    }
    start_time = millis() ;
  }
  ext = M5.Axp.isACIN() ;
  if (ext != old_ext)
  {
    old_ext = ext ;
    handle_page4() ;
  }
  M5.lcd.setFont() ;
}
//**************************************************************
void handle_pages(void)
{
  old_page_nr != page_nr ? old_page_nr = page_nr : page_nr ;
  switch (page_nr)         // good for handling more pages (maybe in future)
  {
    case 0 :
      {
        handle_page0() ;  // work page 0 , the N0NBH-bandcolor page
        break ;
      }
    case 1 :
      {
        handle_page1() ;  // work page 1 , the IAP-MUF_table page
        break ;
      }
    case 2 :
      {
        handle_page2() ;  // work page 2 , the N0NBH Text1 page
        break ;
      }
    case 3 :
      {
        handle_page3() ;  // work page 3 , the N0NBH Text2 page
        break ;
      }
    case 4 :
      {
        handle_page4() ;  // work page 4 , the LiPo info page
        break ;
      }
    default: break ;
  }
}
//***************************************************************
void handle_page0(void)            // side of the band openings
{
  if (!is_basic_pattern)
  {
    draw_basic_pattern() ;              // draw the basic pattern
  }
#ifdef WiFiOn
  get_and_compress_all_client_data() ; // read the main content of the webpage
  get_and_show_bottombar_datas() ;     // create the bottom-bar
  get_and_show_topbar_datas() ;        // create the top-bar
  get_and_show_band_colors() ;         // show band colors
#endif
}
//**************************************************************
void handle_page1(void)      // side of the MUF-table
{
  M5.lcd.setTextSize(2) ;
  String km_string[8] = {" 100", " 200", " 400", " 600", " 800", "1000", "1500", "3000"} ;
  uint16_t g ;
  flag_color = true ;
  new_iap = "" ;                                //state of affairs
  is_basic_pattern = false ;                    // flag for page0
  M5.lcd.fillScreen(BLACK) ;                       // clear blue screen
  M5.lcd.fillRoundRect(0, 0, 240, 16, 5, MAROON) ; // build top header
  M5.lcd.drawRoundRect(0, 0, 240, 17, 5, WHITE) ;
  M5.lcd.setTextColor(CYAN) ;
  read_iap_datas() ;                            // get the muf-table-datas
  M5.lcd.setCursor(0, 5) ;
  M5.lcd.setTextSize(1) ;
  M5.lcd.print(" " + content_strings3[10] + " UTC ") ; //show the last change-time
  M5.lcd.setTextColor(WHITE) ;
  M5.lcd.print(" tnx:iap_kborn.de") ;
  M5.lcd.setTextSize(2) ;
  M5.lcd.fillRoundRect(3, 19, 234, 298, 5, B_DD6USB + 12) ;// background
  M5.lcd.drawRoundRect(3, 19, 234, 298, 5, YELLOW) ; // generate the borders
  M5.lcd.setCursor(7, 28) ;                          // headline position
  M5.lcd.setTextColor(GREEN) ;                       // headline color
  M5.lcd.print("Juliusruh MUF-Table") ;              // generate headline
  M5.lcd.setTextColor(WHITE) ;
  M5.lcd.setCursor(0, 55) ;
  M5.lcd.println("     Km      MHz") ;// table header
  M5.lcd.setTextColor(YELLOW) ;
  for (g = 1 ; g < 9 ; g++)
  {
    M5.lcd.setCursor(18, 56 + g * 29) ;             // startposition of table-text
    M5.lcd.print("  " + km_string[g - 1]) ;
    M5.lcd.setCursor(145 , 56 + g * 29) ;
    if (content_strings3[8 - g].toInt() < 10) M5.lcd.print(" ") ; // arrange positions
    M5.lcd.println(content_strings3[8 - g]) ;     // print the MUF-values
  }
  M5.lcd.drawFastVLine(115, 53, 256, CYAN) ;      // build column-bar
  M5.lcd.drawRect(11, 50, 218, 262, CYAN) ;       // generate table-frame
  for (g = 77 ; g < 300 ; g += 29)                // build the table-rows now
  {
    M5.lcd.drawFastHLine(12, g , 216, CYAN) ;
  }
}
//**************************************************************
void handle_page2(void)   // first side of the solar datas
{
  char * content_strings1[] = { "<solarflux>", "<aindex>", "<kindex>",
                                "<kindexnt>", "<xray>", "<sunspots>",
                                "<heliumline>", "<protonflux>", "<electonflux>",
                                "<aurora>", "<normalization>", "<latdegree>"
                              } ;    // names of content (part 1)
  flag_color = true ;                // flag for changing colors
  M5.lcd.setFont() ;
  M5.lcd.fillScreen(B_DD6USB + 7);   // background for the page
  is_basic_pattern = false ;         // flag for page0
  M5.lcd.fillRoundRect(0, 0, 240, 20, 5, MAROON) ;
  M5.lcd.setCursor(180, 5) ;         // generate the top bar
  M5.lcd.setTextColor(GREEN) ;
  M5.lcd.setTextSize(1) ;
  M5.lcd.print("tnx:N0NBH") ;
  get_and_show_topbar_datas() ;      // show top bar
  M5.lcd.setFont(&FreeSans9pt7b) ;
  M5.lcd.setCursor(0, 40) ;          // start displaying the sun values
  color = GREEN ;
  M5.lcd.setTextColor(color) ;
  for (uint8_t i = 0 ; i < 12 ; i++) // list the content (part 1)
  {
    parse_string_and_tftprint(content_strings1[i]) ;
  }
  M5.lcd.setTextColor(WHITE) ;
  M5.lcd.setCursor(0, 307) ;
  M5.lcd.print("===>>  continued next page") ; //hint for page 2
  flag_color = false ;
}
//***************************************************************
void handle_page3(void)   // second side of the solar datas
{
  char * content_strings2[] = {"<solarwind>", "<magneticfield>", "<geomagfield>",
                               "<signalnoise>", "<fof2>", "<muffactor>", "<muf>"
                              } ;   // names of content (part 2)
  flag_color = true ;                 // flag for changing colors
  M5.lcd.setFont() ;
  M5.lcd.fillScreen(B_DD6USB + 7);        // background for the page
  is_basic_pattern = false ;           // flag for page0
  M5.lcd.fillRoundRect(0, 0, 240, 20, 5, MAROON) ;
  M5.lcd.setCursor(180, 5) ;             // generate the top bar
  M5.lcd.setTextColor(GREEN) ;
  M5.lcd.print("tnx:N0NBH") ;             // thanks PAUL !
  get_and_show_topbar_datas() ;        // show top bar
  M5.lcd.setFont(&FreeSans9pt7b) ;
  M5.lcd.setCursor(0, 43) ;               // start display of the sun values
  color = GREEN ;
  M5.lcd.setTextColor(color) ;
  for (uint8_t i = 0 ; i < 7 ; i++)    // list the content (part 2)
  {
    parse_string_and_tftprint(content_strings2[i]) ;
  }
  M5.lcd.print("\n   =====>>  EOM  <<=====") ;  // END OF MESSAGE !
  M5.lcd.setFont() ;
  M5.lcd.fillRoundRect(0, 300, 240, 20, 5, MAROON) ;
  M5.lcd.drawRoundRect(0, 300, 240, 20, 5, WHITE) ;
  M5.lcd.setTextColor(CYAN) ;
  M5.lcd.setCursor(3, 306) ;
  M5.lcd.setTextColor(WHITE) ;
  M5.lcd.setTextSize(0) ;
  M5.lcd.print(V_String) ;           // show version-number
  flag_color = false ;
}
//***************************************************************
void handle_page4(void)
{
 float valx ;
  M5.lcd.setFreeFont(&FreeSansBold12pt7b) ;
  M5.lcd.fillScreen(BLUE) ;
  M5.lcd.fillRoundRect(45, 24, 150, 35, 10, MAROON) ;
  M5.lcd.drawRoundRect(44, 23, 152, 37, 10, WHITE) ;
  M5.lcd.setTextColor(YELLOW) ;
  M5.lcd.setTextSize(1) ;
  M5.lcd.setCursor(60, 50) ;
  M5.lcd.print("LiPo INFO") ;
  M5.lcd.setTextColor(WHITE) ;
  M5.lcd.setFreeFont(&FreeSans9pt7b) ;
  M5.lcd.setCursor(5, 100) ;
  M5.lcd.print("  is ext. power ?         ") ;
  ext = M5.Axp.isACIN() ;
  ext ? M5.lcd.println("YES") : M5.lcd.println("NO") ;
  M5.lcd.setCursor(5, M5.lcd.getCursorY() + 10) ;
  M5.lcd.print("  is charging    ?         ") ;
  valx = M5.Axp.isCharging() ;
  while (valx == 0 && ext)
  {
    valx = M5.Axp.isCharging() ;
  }
  M5.Axp.isCharging() ? M5.lcd.println("YES") : M5.lcd.println("NO") ;
  M5.lcd.setCursor(5, M5.lcd.getCursorY() + 10) ;
  M5.lcd.print("  charging current   ") ;
  valx = M5.Axp.GetBatChargeCurrent() ;
  while (valx == 0 && ext)
  {
    valx = M5.Axp.GetBatChargeCurrent() ;
  }
  if (valx == 0) M5.lcd.print("  ") ;
  M5.lcd.print(valx, 1) ;
  M5.lcd.println(" mA") ;
  M5.lcd.setCursor(5, M5.lcd.getCursorY() + 10) ;
  M5.lcd.print("  discharging curr. ") ;
  valx = M5.Axp.GetBatPower() ;
  if (valx == 0) M5.lcd.print("    ") ;
  M5.lcd.print(valx, 1) ;
  M5.lcd.println(" mA") ;
  valx = M5.Axp.GetBatVoltage() ;
  M5.lcd.setCursor(5, M5.lcd.getCursorY() + 10) ;
  M5.lcd.print("  LiPo voltage            ") ;
  M5.lcd.print(valx, 1) ;
  M5.lcd.println("  V") ;
  Percentage = ( valx < 3.2 ) ? 0 : ( valx - 3.2 ) * 100 ;
  M5.lcd.setCursor(5, M5.lcd.getCursorY() + 10) ;
  M5.lcd.print("  LiPo loading          ") ;
  M5.lcd.print(Percentage, 1) ;
  M5.lcd.println("  %") ;
  M5.lcd.setFont() ;
  // now show the loading-bar
  M5.lcd.fillRoundRect(8, 280, 227, 26, 5, BLACK) ;
  bool flag = false ;
  uint16_t bar = map(int(Percentage), 0, 100, 0, 223) ;
  M5.lcd.setTextColor(WHITE) ;
  M5.lcd.setTextSize(1) ;
  switch (int(Percentage))
  {
    case 0 ... 69 :
      {
        if (!flag)
        {
          M5.lcd.fillRect(90, 283, 80, 20, BLACK) ;
          flag = !flag ;
        }
        M5.lcd.fillRoundRect(10, 284, bar, 17, 5, GREENYELLOW) ;
        M5.lcd.setCursor(168, 289) ;
        M5.lcd.printf("%.1f", Percentage) ;
        delay(100) ;
        break ;
      }
    default :
      {
        M5.lcd.setTextColor(BLACK, GREENYELLOW) ;
        M5.lcd.fillRoundRect(10, 284, bar, 17, 5, GREENYELLOW) ;
        M5.lcd.setCursor(110, 289) ;
        M5.lcd.printf("%.1f", Percentage) ;
        delay(100) ;
        break ;
      }
  }
}
//**************************************************************
void read_iap_datas(void)
{
#ifdef WiFiOn
  if ((wifiMulti.run() == WL_CONNECTED))
  {
    HTTPClient http ;
#ifdef debug
    Serial.print("[HTTP] begin...\n") ;    // configure server and url
#endif
    http.begin("https://www.ionosonde.iap-kborn.de/latest_JR055_MUF.txt") ; //HTTP
    int httpCode = http.GET() ;
    if (httpCode > 0)    // httpCode will be negative on error
    {
      // HTTP header has been send and Server response header has been handled
#ifdef debug1
      Serial.printf("[HTTP] GET... code: %d\n", httpCode) ;
#endif
      if (httpCode == HTTP_CODE_OK)     // server send "300"
      {
        iap_buf = http.getString() ;
      }
    }
    else
    {
#ifdef debug1
      Serial.printf("[HTTP] GET... failed, error: %s\n", http.errorToString(httpCode).c_str());
#endif
    }
    http.end() ;
    //===============>>  now read the webpage  <<==================
#ifdef debug
    Serial.println("===========================================") ;
    Serial.print("IAP_buf = " + iap_buf) ;
    Serial.println("===========================================") ;
#endif
    new_iap = "" ;
    for (uint8_t z = 0 ; z < 8 ; z++)            // parse iap_content
    {
      pos = iap_buf.lastIndexOf("\n") ;          // search for every newline
      t = iap_buf.substring(pos - 5 , pos) ;     // ectract substring
      iap_buf = iap_buf.substring(0, pos - 5 ) ; // build iap_substring
      t.trim() ;                                 // delete whitespace
      content_strings3[z] = t ;                  // create iap_fullstring
      new_iap = new_iap + t ;
#ifdef debug
      Serial.println("\n " + content_strings3[z]) ;
#endif
    }
    if (old_iap = "") old_iap = new_iap ;        // first initialization of old_iap
    content_strings3[10] = iap_buf.substring(16, 32) ; // len = 16 holds: date & time
    content_strings3[10].setCharAt(4, '/') ;           // change single signs
    content_strings3[10].setCharAt(7, '/') ;
    content_strings3[10].setCharAt(10, ' ') ;
    content_strings3[10].setCharAt(13, ':') ;
#ifdef debug
    Serial.println(content_strings3[10]) ;
#endif
  }
#endif
}
//**************************************************************
void parse_string_and_tftprint(String s11)   // side of the band openings
{
  t2 = "</" + s11.substring(1, s11.length()) ;// parse content-strings
  pos  = pbuf.indexOf(s11) + s11.length() ;
  pos1 = pbuf.indexOf(t2) ;
  t2 = ("  " + s11.substring(1, s11.length() - 1) + " = ") ;
  if (t2 == "  electonflux = ")
  {
    t2 = "  electronflux = " ;
  }
  if (flag_color)                             // show on screen ?
  {
    M5.lcd.print(t2) ;
    t = pbuf.substring(pos, pos1) ;
    if ((t.length() == 0) || (t == "NoRpt"))
    {
      t = "NoReport" ;
    }
    M5.lcd.println(t) ;
    color == GREEN ? color = YELLOW : color = GREEN ;
    M5.lcd.setTextColor(color) ;          // change text color Green<=>Yellow
  }
}
//***************************************************************
bool time_loop_finished(void)
{
  if ((start_time + loop_time) > millis())
  {
    return false ;
  }
  else
  {
    return true ;
  }
}
//***************************************************************
void draw_basic_pattern(void)
{
  M5.lcd.fillScreen(B_DD6USB + 8) ; // a little more intense
  draw_WiFi_Quality() ;             // draw top bar WiFi
  draw_band_pattern() ;
  is_basic_pattern = true ;
}
//***************************************************************
void draw_band_pattern(void)
{
  for ( int i = 18; i < 280; i += 40)
  {
    M5.lcd.drawFastHLine(0, i, 240, YELLOW) ;
  }
  M5.lcd.drawFastVLine(170, 20, 280, YELLOW) ;
  M5.lcd.drawFastVLine(105, 20, 200, YELLOW) ;
  M5.lcd.drawFastVLine( 0, 20, 280, YELLOW) ;
  M5.lcd.drawFastVLine(239, 20, 280, YELLOW) ;
  M5.lcd.setTextSize(2) ;
  M5.lcd.setCursor(28, 33) ;
  M5.lcd.setTextColor(YELLOW) ;
  M5.lcd.print("Band") ;
  M5.lcd.setTextSize(2) ;
  M5.lcd.setCursor(12, 73) ;
  M5.lcd.print("80m-40m") ;
  M5.lcd.setCursor(12, 113) ;
  M5.lcd.print("30m-20m") ;
  M5.lcd.setCursor(12, 153) ;
  M5.lcd.print("17m-15m") ;
  M5.lcd.setCursor(12, 193) ;
  M5.lcd.print("12m-10m") ;
  M5.lcd.setCursor(8, 233) ;
  M5.lcd.print("Eskip 6m [EU]") ;
  M5.lcd.setCursor(8, 273) ;
  M5.lcd.print("Eskip 4m [EU]") ;
  M5.lcd.setTextSize(2) ;
  M5.lcd.setTextColor(WHITE) ;
  M5.lcd.setCursor(118, 33) ;
  M5.lcd.print("day") ;
  M5.lcd.setTextColor(CYAN) ;
  M5.lcd.setCursor(176, 33) ;
  M5.lcd.print("night") ;
}
//***************************************************************
int8_t getWifiQuality() // converts the dBm to a range between 0 and 100%
{
  int32_t dbm = WiFi.RSSI() ; // get WiFi RSSI
  if (dbm <= -100)
  {
    return 0 ;
  } else if (dbm >= -50)
  {
    return 100;
  } else
  {
    return 2 * (dbm + 100);
  }
}
//******************************************************************
void draw_WiFi_Quality(void)
{
  M5.lcd.setTextSize(0) ;
  char buf[10] ;
  int8_t quality = getWifiQuality() ;
  M5.lcd.setTextColor(GREEN) ;
  M5.lcd.setCursor(170, 5) ;
  sprintf(buf, "RSSI:%2d%c", quality, '%') ;
  M5.lcd.print(buf) ;
  for (int8_t i = 0; i < 4; i++)
  {
    for (int8_t j = 0; j < 2 * (i + 1); j++)
    {
      if (quality > i * 25 || j == 0)
      {
        M5.lcd.drawPixel(224 + 2 * i, 12 - j, YELLOW);
      }
    }
  }
}
//******************************************************************
void get_and_compress_all_client_data(void)
{
  pbuf = "" ;                      // clear pbuf
  String  sbuf = "" ;
  HTTPClient http;
  http.begin("https://www.hamqsl.com/solarxml.php") ;
  // start connection and send HTTP header
  int httpCode = http.GET() ;
  if (httpCode > 0)      // httpCode will be negative on error
  {
    // HTTP header has been send and Server response header has been handled
#ifdef debug
    Serial.printf("[HTTP] GET...returns code: %d\n", httpCode);
#endif
    // file found at server
    if (httpCode == HTTP_CODE_OK)
    {
      sbuf = http.getString();
    }
  } else
  {
    Serial.printf("[HTTP] GET... failed, error: %s\n", http.errorToString(httpCode).c_str());
  }
  http.end();
#ifdef debug
  Serial.printf(" len sbuf = %u  ==>> sbuf = %s", sbuf.length(), sbuf) ;
  //  Serial.println(pbuf.length()) ;
  //  Serial.println("     ================>>>>    sbuf = ") ;
  Serial.println(sbuf) ;
#endif
  //===============>>  now compress the content  <<==================

  // means: delete all: cr,lf,",space,tab and shortening the string
  String t  = "<updated>" ; // begin of "interest" of the content
  String t1 = "</muf>" ;    //  end  of "interest" of the content
  for (int j = (sbuf.indexOf(t) + t.length()) ; j < sbuf.indexOf(t1) ; j++)
  {
    atom = sbuf[j] ;
    if (!isWhitespace(atom) && !isControl(atom) && (atom != 34))
    {
      pbuf = pbuf + atom ;     // build the compressed string
#ifdef debug1
      Serial.print(atom) ;
      delay(50) ;
#endif
    }
  }
  // in pbuf is now the compessed content-ready for parsing
#ifdef debug
  Serial.println() ;
  Serial.println(pbuf) ;
  Serial.print(" len pbuf = ") ;
  Serial.println(pbuf.length()) ;
#endif
}
//*******************************************************************
void get_and_show_bottombar_datas(void)
{
  char * content_string[] = {"<solarflux>", "<signalnoise>", "<muf>"} ;
  String Flux, SN, Muf ;
  char kbuf[40] ;
  flag_color = false ;
  M5.lcd.setFont() ;
  M5.lcd.setTextSize(1) ;
  M5.lcd.fillRoundRect(0, 300, 240, 20, 5, MAROON) ;
  M5.lcd.drawRoundRect(0, 300, 240, 20, 5, WHITE) ;
  M5.lcd.fillRoundRect(1, 302, 57, 16, 5, BLACK) ;
  M5.lcd.setTextColor(CYAN) ;
  M5.lcd.setCursor(2, 306) ;
  M5.lcd.print("tnx:N0NBH") ;
  parse_string_and_tftprint(content_string[0]) ;
  Flux = pbuf.substring(pos, pos1) ;
  parse_string_and_tftprint(content_string[1]) ;
  SN = pbuf.substring(pos, pos1) ;
  parse_string_and_tftprint(content_string[2]) ;
  Muf = pbuf.substring(pos, pos1) ;
  if (isAlpha(Muf.charAt(1)))
  {
    Muf = "noInfo" ;
  }
  M5.lcd.setCursor(51, 306) ;
  M5.lcd.setTextColor(WHITE) ;
  sprintf(kbuf, "  Flux:%s S/N:%s MUF:%s", Flux, SN, Muf) ;
  M5.lcd.print(kbuf) ;
#ifdef debug1
  Serial.printf"kbuf = %s", kbuf) ;
#endif
}
//***************************************************************
void get_and_show_topbar_datas(void)
{
  M5.lcd.setFont() ;
  M5.lcd.setTextSize(1) ;
  M5.lcd.setTextColor(CYAN) ;
  char buf1[30] ;
  String date1 = pbuf.substring(0, 2) ;   // day
  String date2 = pbuf.substring(2, 5) ;   // month
  String date3 = pbuf.substring(5, 9) ;   // year
  String date4 = pbuf.substring(9, 11) ;  // hour
  String date5 = pbuf.substring(11, 13) ; // min
#ifdef debug
  Serial.println("string-parts:" + date1 + "," + date2 + "," + date3 + "," + date4 + "," + date5) ;
#endif
  sprintf(buf1, "from %s %s %s %s:%s UTC", date1, date2, date3, date4, date5) ;
  M5.lcd.setCursor(5, 5) ;
  M5.lcd.print(buf1) ;
  M5.lcd.drawRoundRect(0, 0, 240, 18, 5, WHITE) ;
}
//******************************************************************
int check_color(String)  // colors of the Ham-band-areas
{
  if (t == "P")          // check "poor"
  {
    color = RED ;
  }
  else if ( t == "F")   // check "fair"
  {
    color = YELLOW ;
  }
  else if ( t == "G")   // check "good"
  {
    color = GREEN ;
  }
  return (color) ;
}
//********************************************************************
int check_day_80_40m(void)
{
  t = "40mtime=day>" ;
  x = pbuf.indexOf(t) + t.length();
  t = pbuf.substring(x, x + 1) ;      // one letter is significant
  return (check_color(t)) ;
}
//********************************************************************
int check_night_80_40m(void)
{
  t = "40mtime=night>" ;
  x = pbuf.indexOf(t) + t.length() ;
  t = pbuf.substring(x, x + 1) ;
  return (check_color(t)) ;
}
//********************************************************************
int check_day_30_20m(void)
{
  t = "20mtime=day>" ;
  x = pbuf.indexOf(t) + t.length() ;
  t = pbuf.substring(x, x + 1) ;
  return (check_color(t)) ;
}
//********************************************************************
int check_night_30_20m(void)
{
  t = "20mtime=night>" ;
  x = pbuf.indexOf(t) + t.length() ;
  t = pbuf.substring(x, x + 1) ;
  return (check_color(t)) ;
}
//********************************************************************
int check_day_17_15m(void)
{
  t = "15mtime=day>" ;
  x = pbuf.indexOf(t) + t.length() ;
  t = pbuf.substring(x, x + 1) ;
  return (check_color(t)) ;
}
//********************************************************************
int check_night_17_15m(void)
{
  t = "15mtime=night>" ;
  x = pbuf.indexOf(t) + t.length() ;
  t = pbuf.substring(x, x + 1) ;
  return (check_color(t)) ;
}
//********************************************************************
int check_day_12_10m(void)
{
  t = "10mtime=day>" ;
  x = pbuf.indexOf(t) + t.length() ;
  t = pbuf.substring(x, x + 1) ;
  return (check_color(t)) ;
}
//********************************************************************
int check_night_12_10m(void)
{
  t = "10mtime=night>" ;
  x = pbuf.indexOf(t) + t.length() ;
  t = pbuf.substring(x, x + 1) ;
  return (check_color(t)) ;
}
//********************************************************************
int check_6m(void)
{
  t = "_6m>" ;
  x = pbuf.indexOf(t) + t.length() ;
  t = pbuf.substring(x, x + 1) ;
  return (check_color(t)) ;
}
//********************************************************************
int check_4m(void)
{
  t = "_4m>" ;
  x = pbuf.indexOf(t) + t.length() ;
  t = pbuf.substring(x, x + 1) ;
  return (check_color(t)) ;
}
//********************************************************************
void  get_and_show_band_colors(void)
{
  M5.lcd.fillRect(107, 61, 62, 35, check_day_80_40m()) ;    // 80 day
  M5.lcd.fillRect(173, 61, 64, 36, check_night_80_40m()) ;  // 80 night
  M5.lcd.fillRect(107, 101, 62, 35, check_day_30_20m()) ;   // 20 day
  M5.lcd.fillRect(173, 101, 64, 35, check_night_30_20m()) ; // 20 night
  M5.lcd.fillRect(107, 141, 62, 35, check_day_17_15m()) ;   // 15 day
  M5.lcd.fillRect(173, 141, 64, 35, check_night_17_15m()) ; // 15 night
  M5.lcd.fillRect(107, 181, 62, 35, check_day_12_10m()) ;   // 10 day
  M5.lcd.fillRect(173, 181, 64, 35, check_night_12_10m()) ; // 10 night
  M5.lcd.fillRect(173, 221, 64, 35, check_6m()) ;           //  6
  M5.lcd.fillRect(173, 261, 64, 37, check_4m()) ;           //  4
}
//********************************************************************
bool check_N0NBH_datas(void)
{
  old_pbuf = pbuf ;                       // hold the last values
  //  get_and_compress_all_client_data() ;    // get the present values (only for evaluating)
  if (old_pbuf.substring(20, old_pbuf.length()) == pbuf.substring(20, pbuf.length()))
  {
#ifdef debug
    Serial.println("N0NBH-Strings are equal") ;
#endif
    return true ;
  }
  else
  {
#ifdef debug
    Serial.println("N0NBH-Strings not equal !!") ;
#endif
    old_page_nr  = -1 ;            //force updating of the present page
    return false ;
  }
}
//***************************************************************
void handle_OTA(void)
{
  //ArduinoOTA.setPort(8266) ;                     // Port defaults to 8266
  ArduinoOTA.setHostname("M5Stack for Solardat") ; // Hostname defaults to esp8266-[ChipID]
  // ArduinoOTA.setPassword((const char *)"123") ; // No authentication by default
  ArduinoOTA.onStart([]()
  {
#ifdef debug
    Serial.println("Start") ;
#endif
  } ) ;
  ArduinoOTA.onEnd([]()
  {
#ifdef debug
    Serial.println("End") ;
#endif
  } ) ;
  ArduinoOTA.onProgress([](unsigned int progress, unsigned int total)
  {
#ifdef debug
    Serial.printf("Progress: %u%%\n", (progress / (total / 100))) ;
#endif
  } ) ;
  ArduinoOTA.onError([](ota_error_t error)
  {
#ifdef debug
    Serial.printf("Error[%u]: ", error) ;
    if (error == OTA_AUTH_ERROR) Serial.println("Auth Failed") ;
    else if (error == OTA_BEGIN_ERROR) Serial.println("Begin Failed") ;
    else if (error == OTA_CONNECT_ERROR) Serial.println("Connect Failed") ;
    else if (error == OTA_RECEIVE_ERROR) Serial.println("Receive Failed") ;
    else if (error == OTA_END_ERROR) Serial.println("End Failed") ;
#endif
  } ) ;
  ArduinoOTA.begin() ;
#ifdef debug
  Serial.println("OTA READY") ;
  Serial.println("Ready") ;
  Serial.print("IP address: ") ;
  Serial.println(WiFi.localIP()) ;
#endif
}
//***************************************************************************
void check_voltage(void)
{
  float threshold = M5.Axp.GetBatVoltage() ;
  if (threshold < 3.2 )
  {
    M5.Axp.PowerOff() ;                // if LiPo discharged : switch device off !!!
  }
}
//************************************************
